import unittest
from unittest.mock import MagicMock, patch
from interface_openstack.implementation.vm.instance import OSVirtualMachineManager

servers = [
    MagicMock(
        id="test1",
        name="test1",
        flavorRef="flav1",
        imageRef="image1",
        networks=[{"uuid": "network1"}],
        terminated_at="20200320-15-31",
        launched_at="20190510-13-22",
        disks=["disk1"],
        status="ACTIVE",
        addresses=[{"mac": "12345678AB", "ipv4": "1.1.1.1"}]
    )
]

fake_auth = {
    "auth_url":  "",
    "project_id":  "",
    "project_name": "",
    "user_domain_name":  "",
    "user_domain_id":  "",
    "project_domain_id":  "",
    "username":  "",
    "password":  "",
    "region_name":  "",
    "interface":  "",
    "identity_api_version":  "",
}


class InstanceCreateTestCase(unittest.TestCase):
    '''
        Test openstack methods called with the right values or exceptions raised.

        Signature of tests:

        def test_my_feature_with(self):
            # Mock
            --- mocking openstack compute API ---
            # Execute
            --- execute the tested method ---
            # Assert
            --- check values, exceptions, calls etc. ---
    '''

    @patch('openstack.connect')
    def setUp(self, mock_connect):
        self.compute = MagicMock()
        self.compute.get_server = MagicMock(return_value=servers[0])
        mock_connect().compute = self.compute
        self.manager = OSVirtualMachineManager(auth=fake_auth)

    def tearDown(self):
        pass

    def test_create_from_template(self):
        # Mock
        self.compute.create_server = MagicMock(return_value=servers[0])
        self.compute.find_flavor = MagicMock(return_value=MagicMock(id="uuid1"))
        self.compute.find_image = MagicMock(return_value=MagicMock(id="uuid2"))

        # Execute
        instance_id = self.manager.create_vm_from_template('test1',
                                                           'imageid',
                                                           'flavorid',
                                                           ['networkid1'])

        # Assert
        self.manager.openstack.compute.create_server.assert_called()
        self.assertEqual(instance_id, servers[0].id)

    def test_create_from_template_params(self):
        # Mock
        self.compute.create_server = MagicMock(return_value=servers[0])
        self.compute.find_flavor = MagicMock(return_value=MagicMock(id="uuid1"))
        self.compute.find_image = MagicMock(return_value=MagicMock(id="uuid2"))

        # Execute
        self.manager.create_vm_from_template('test', 'imageid', 'flavorid',
                                             ['networkid1'])
        # Assert
        self.manager.openstack.compute.create_server.assert_called_with(
                                                    name='test',
                                                    flavorRef="uuid1",
                                                    imageRef="uuid2",
                                                    networks=['networkid1'],
                                                    )

    def test_suspend(self):
        # Mock
        self.compute.suspend_server = MagicMock(return_value=True)
        # Execute
        self.manager.suspend_vm("test1")
        # Assert
        self.compute.get_server.assert_called()
        self.compute.suspend_server.assert_called_once()

    def test_start(self):
        # Mock
        self.compute.start_server = MagicMock(return_value=True)
        # Execute
        self.manager.start_vm("test1")
        # Assert
        self.compute.get_server.assert_called()
        self.compute.start_server.assert_called_once()

    def test_stop(self):
        # Mock
        self.compute.stop_server = MagicMock(return_value=True)
        # Execute
        self.manager.stop_vm("test1")
        # Assert
        self.compute.get_server.assert_called()
        self.compute.stop_server.assert_called_once()

    def test_wake_up(self):
        # Mock
        self.compute.resume_server = MagicMock(return_value=True)
        # Execute
        self.manager.wake_up_vm("test1")
        # Assert
        self.compute.get_server.assert_called()
        self.compute.resume_server.assert_called_once()

    def test_reboot(self):
        # Mock
        self.compute.reboot_server = MagicMock(return_value=True)
        # Execute
        self.manager.reboot_vm("test1")
        # Assert
        self.compute.get_server.assert_called()
        self.compute.reboot_server.assert_called_once()
        self.compute.reboot_server.assert_called_with(servers[0], reboot_type="SOFT")

    def test_reset(self):
        # Mock
        self.compute.reboot_server = MagicMock(return_value=True)
        # Execute
        self.manager.reset_vm("test1")
        # Assert
        self.compute.get_server.assert_called()
        self.compute.reboot_server.assert_called_once()
        self.compute.reboot_server.assert_called_with(servers[0], reboot_type="HARD")
